import os, re
from bs4 import BeautifulSoup
import functions

import markdown

class Index:
    def __init__(self,indexfile):
        self.pages = []
        self.fixlinks = False

        f = open(indexfile).read()
        self.root = BeautifulSoup(f)
        self.toc = self.table_of_contents(self.root)
        self.makelist(self.toc,0)

        self.title = self.title_index(self.root)
        print self.markdown(ordered=True, mindepth=2)
        print markdown.markdown(self.markdown(ordered=True, mindepth=2))

    def table_of_contents(self, node):
        functions.remove_tag(node,'div',id='footer')
        functions.remove_tag(node,'div','pageSection','main-content')
        functions.remove_tag(node,'div','pageSectionHeader')
        functions.remove_tag(node,'div',id='main-header')
        functions.remove_tag(node,'img')
        return node.find('div','pageSection').ul

    def add(self, item):
        self.pages.append(item)

    def clear(self):
        del self.pages[:]

    def makelist(self, item, depth, char='*'):
        node = {'name': item.a.text, 'depth': depth, 'url': item.a['href']}
        self.pages.append(node)

        for x in item.li.find_all('ul',recursive=False):
            self.makelist(x, depth+1, char)

    def configure(self, fixlinks=False):
        self.fixlinks = fixlinks

    def title_index(self, node):
        output = node.find('title').text
        output = re.sub('.*\((.*)\)','\g<1>', output)
        return output.lstrip()

    def pagelist(self):
        return self.pages
            
    def markdown(self, mindepth=0, ordered=False):
        output = ""
        if ordered:
            char = '1.'
        else:
            char = '*'

        if self.fixlinks:
            h = '#'
        else:
            h = ''

        for i in self.pages:
            if i['depth'] >= mindepth:

                output += ' '*(i['depth']-mindepth)*4 + char + ' '
                if self.fixlinks:
                    filename = i['url']
                else:
                    filename = os.path.splitext(i['url'])[0]+'.md'
                output += self.link(i['name'], h+filename)
                output += '\n'

        return output

    def html(self, mindepth=0, ordered=False):
        return markdown.markdown(self.markdown(ordered=ordered, mindepth=mindepth))

    def link(self, name, url):
        output = ""
        output += '['+name+']'
        output += '('+url+')'
        return output



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys, re
import codecs

import argparse
import getpass

import xmlrpclib
import requests

import zipfile

from parsers import html, pageindex

import webbrowser
import urlparse, urllib
from bs4 import BeautifulSoup

def from_scriptroot(filename):
    currentpath = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(currentpath,filename)

def create_dir(path):
    if not os.path.exists(path):
            os.makedirs(path)


def download_file(url, auth):
    local_filename = url.split('/')[-1]

    with open(local_filename, 'wb') as f:
        print "Downloading from: %s" % (url)
        r = requests.get(url, auth=auth, stream=True)
        total_length = r.headers.get('content-length')

        if total_length is None:
            f.write(r.content)
        else:
            dl = 0
            total_length = int(total_length)
            for data in r.iter_content(chunk_size=4096):
                dl += len(data)
                f.write(data)
                done = int(50 * dl / total_length)
                sys.stdout.write("\r[%s%s]" % ('=' * done, ' ' * (50 - done)) )
                sys.stdout.flush()

    sys.stdout.write(" -> %s\n" % (local_filename) )
    return local_filename


def export_space(url, key):
    proxy = xmlrpclib.ServerProxy(url+"/rpc/xmlrpc")
    
    user = raw_input("Enter Confluence username: ")
    passwd = getpass.getpass("Enter Confluence password: ")
    auth = (user, passwd)

    token = proxy.confluence2.login(user,passwd)
    
    space = proxy.confluence2.getSpace(token, key) 
    print "Exporting space: %s Key: %s" % (space['name'], space['key'])
    spaceurl = proxy.confluence2.exportSpace(token,key,"TYPE_HTML")

    proxy.confluence2.logout(token)

    return download_file(spaceurl, auth)

def unzip_space(filename):

    print "Verifying %s" % (filename)
    z = zipfile.ZipFile(filename,'r')
    z.testzip()
    z.extractall()
    z.close()

def write_last(text):
    f = open('last','w')
    f.write(text)
    f.close()

def read_last():
    f = open('last','r')
    text = f.read()
    f.close()
    return text

def console():
    parser = argparse.ArgumentParser(description='Export a Confluence Space')

    parser.add_argument('-u','--url', nargs=1, metavar='URL', help='Download space first from server URL')
    parser.add_argument('-s','--space', nargs=1, metavar='SPACE', required=True, help='Space key')
    parser.add_argument('--single-page', action='store_true', help='Render output to single page (auto-set by --pdf)')
    parser.add_argument('--pdf', action='store_true', help='Render output to PDF')
    parser.add_argument('--toc', action='store_true', help='Add table of contents to single-page document')

    args = parser.parse_args()

    home = os.getcwd()

    create_dir("build")
    os.chdir("build")
    write_last(args.space[0])

    create_dir(args.space[0])
    os.chdir(args.space[0])

    if args.url:
        r = export_space(args.url[0], args.space[0])
        write_last(r)
        unzip_space(r)

    last = read_last()
    create_dir(args.space[0])
    os.chdir(args.space[0])

    if args.pdf or args.single_page:
        singlepage = True
    else:
        singlepage = False

#    print singlepage

    index = pageindex.Index('index.html')
    index.configure(fixlinks=singlepage)
    text = index.markdown(mindepth=1)
    print index.title
    g = codecs.open('index.md','w',encoding='utf-8')
    g.write(text)
    g.close()

#    print index.pagelist()

    for i in index.pagelist():
        filename = i['url']
        f = codecs.open(filename, encoding='utf-8').read()
        s = BeautifulSoup(f)

        l = i['depth']+1
        output = html.page(s,depth=l-1,singlepage=singlepage).prettify()
        output = '<h'+str(l)+' id="'+i['url']+'">'+i['name']+'</h'+str(l)+'>\n' + output

        newname = os.path.splitext(filename)[0]+'.md'
        f = codecs.open(newname,'w',encoding='utf-8')
        f.write(output)
        f.close()

        #print output

    spacename = args.space[0]+'_'+index.title.replace(' ','_')
    spacename = os.path.join(home, spacename)

    if singlepage:
        f = codecs.open('_output_.md','w',encoding='utf-8')

        if args.toc:
            f.write('<h1>Contents</h1>\n')
            f.write(index.html(mindepth=1))
            f.write(html.pagebreak())

        for i in index.pagelist():
            filename = os.path.splitext(i['url'])[0]+u'.md'
            text = codecs.open(filename,'r',encoding='utf-8').read()+u'\n'
            text = text + html.pagebreak()
            f.write(text)

        f.close()

    if args.pdf:
        import weasyprint
        src = weasyprint.HTML(filename='_output_.md')
        doc = src.render(stylesheets=[from_scriptroot("default.css")])
        doc.resolve_links()
    #    for d in doc.pages:
    #        print d.links
        doc.write_pdf(spacename+'.pdf')



from bs4 import BeautifulSoup

def remove_tag(root, tag, class_=None, id=None):
    if class_ == None and not id == None:
        res = root.find_all(tag, id=id)
    if not class_ == None and id == None:
        res = root.find_all(tag, class_=class_)
    if not class_ == None and not id == None:
        res = root.find_all(tag, class_=class_, id=id)
    if class_ == None and id == None:
        res = root.find_all(tag)

    for h in res:
        h.decompose()




import os, re
from bs4 import BeautifulSoup

import pageindex 
import functions

def image(tag):
    src = tag.attrs['src']
    if 'width' in tag.attrs:
        tag.attrs = {'width':tag.attrs['width']}
    else:
        tag.attrs = {}

    tag.attrs['src'] = src

def fix_anchors(node):
    for a in node.find_all('a'):
        if 'href' in a:
            if os.path.isfile(a['href']):
                a['href'] = '#'+a['href']

def fix_links(node):
    for a in node.find_all('a'):
        if os.path.isfile(a['href']):
            a['href'] = os.path.splitext(a['href'])[0]+'.md'


def header(tag, depth, mindepth):
    tag.attrs = {}
    depth = depth - mindepth + int(re.sub('h([1-6])','\g<1>',tag.name))
    depth += 1
    tag.name = 'h'+str(depth)

def pagebreak():
    return '<p style="page-break-after:always !important;"></p>'

def gliffy_diagram(tag):
    pass

def page(node, depth=0, mindepth=0, singlepage=False):
    functions.remove_tag(node,'div',id='footer')
    functions.remove_tag(node,'head')
    node = node.find('div','wiki-content group',id='main-content')

    [x.unwrap() for x in node.find_all('span','confluence-embedded-file-wrapper')]

    [image(x) for x in node.find_all('img')]

    [header(x, depth, mindepth) for x in node.find_all(['h1','h2','h3','h4','h5','h6'])]

    if singlepage:
        fix_anchors(node)
    else:
        fix_links(node)

    return node





from setuptools import setup, find_packages

with open('README.rst') as f:
        long_description = f.read()

#import subprocess
#version = subprocess.check_output(['git','describe','--tags']).strip()
#
setup(
        name = "kyber",
        version = '0.1.0',
        author = "LameStation",
        author_email = "contact@lamestation.com",
        description = "A publishing toolchain for Confluence",
        long_description = long_description,
        license = "GPLv3",
        url = "https://github.com/lamestation/kyber",
        keywords = "confluence documentation generation wiki pdf html export space",
        packages=find_packages(),
        include_package_data=True,
        entry_points={
            'console_scripts': [
                'kyber = kyber.main:console',
                ],
            },
        classifiers=[
            "Environment :: Console",
            "Development Status :: 2 - Pre-Alpha",
            "Topic :: Utilities",
            "Topic :: Documentation",
            "Topic :: Software Development :: Documentation",
            "Topic :: Text Processing :: Markup",
            "Topic :: Text Processing :: Markup :: HTML",
            "Topic :: Utilities",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Programming Language :: Python :: 2 :: Only",
            "Programming Language :: Python :: 2.7",
            ]
        )
